@@ -33,6 +33,7 @@ from calibre.gui2.webengine import (
 from calibre.srv.code import get_translations_data
 from calibre.utils.config import JSONConfig
 from calibre.utils.serialize import json_loads
+from calibre.utils.shared_file import share_open
 from polyglot.builtins import as_bytes, iteritems
 
 try:
@@ -61,6 +62,10 @@ def set_book_path(path, pathtoebook):
     set_book_path.parsed_manifest = json_loads(set_book_path.manifest)
 
 
+def get_manifest():
+    return getattr(set_book_path, 'parsed_manifest', None)
+
+
 def get_path_for_name(name):
     bdir = getattr(set_book_path, 'path', None)
     if bdir is None:
@@ -75,7 +80,7 @@ def get_data(name):
     if path is None:
         return None, None
     try:
-        with lopen(path, 'rb') as f:
+        with share_open(path, 'rb') as f:
             return f.read(), guess_type(name)
     except EnvironmentError as err:
         prints('Failed to read from book file: {} with error: {}'.format(name, as_unicode(err)))
@@ -234,6 +239,8 @@ def create_profile():
 
 class ViewerBridge(Bridge):
 
+    view_created = from_js(object)
+    content_file_changed = from_js(object)
     set_session_data = from_js(object, object)
     set_local_storage = from_js(object, object)
     reload_book = from_js()
@@ -241,12 +248,16 @@ class ViewerBridge(Bridge):
     toggle_bookmarks = from_js()
     toggle_inspector = from_js()
     toggle_lookup = from_js()
+    show_search = from_js()
+    search_result_not_found = from_js(object)
+    find_next = from_js(object)
     quit = from_js()
     update_current_toc_nodes = from_js(object, object)
     toggle_full_screen = from_js()
     report_cfi = from_js(object, object)
     ask_for_open = from_js(object)
     selection_changed = from_js(object)
+    autoscroll_state_changed = from_js(object)
     copy_selection = from_js(object)
     view_image = from_js(object)
     copy_image = from_js(object)
@@ -272,6 +283,7 @@ class ViewerBridge(Bridge):
     goto_frac = to_js()
     trigger_shortcut = to_js()
     set_system_palette = to_js()
+    show_search_result = to_js()
 
 
 def apply_font_settings(page_or_view):
@@ -408,6 +420,9 @@ class WebView(RestartingWebEngineView):
     cfi_changed = pyqtSignal(object)
     reload_book = pyqtSignal()
     toggle_toc = pyqtSignal()
+    show_search = pyqtSignal()
+    search_result_not_found = pyqtSignal(object)
+    find_next = pyqtSignal(object)
     toggle_bookmarks = pyqtSignal()
     toggle_inspector = pyqtSignal()
     toggle_lookup = pyqtSignal()
@@ -416,6 +431,7 @@ class WebView(RestartingWebEngineView):
     toggle_full_screen = pyqtSignal()
     ask_for_open = pyqtSignal(object)
     selection_changed = pyqtSignal(object)
+    autoscroll_state_changed = pyqtSignal(object)
     view_image = pyqtSignal(object)
     copy_image = pyqtSignal(object)
     overlay_visibility_changed = pyqtSignal(object)
@@ -428,12 +444,13 @@ class WebView(RestartingWebEngineView):
     shortcuts_changed = pyqtSignal(object)
     paged_mode_changed = pyqtSignal()
     standalone_misc_settings_changed = pyqtSignal(object)
+    view_created = pyqtSignal(object)
 
     def __init__(self, parent=None):
         self._host_widget = None
         self.callback_id_counter = count()
         self.callback_map = {}
-        self.current_cfi = None
+        self.current_cfi = self.current_content_file = None
         RestartingWebEngineView.__init__(self, parent)
         self.dead_renderer_error_shown = False
         self.render_process_failed.connect(self.render_process_died)
@@ -443,10 +460,15 @@ class WebView(RestartingWebEngineView):
         self._size_hint = QSize(int(w/3), int(w/2))
         self._page = WebPage(self)
         self.bridge.bridge_ready.connect(self.on_bridge_ready)
+        self.bridge.view_created.connect(self.on_view_created)
+        self.bridge.content_file_changed.connect(self.on_content_file_changed)
         self.bridge.set_session_data.connect(self.set_session_data)
         self.bridge.set_local_storage.connect(self.set_local_storage)
         self.bridge.reload_book.connect(self.reload_book)
         self.bridge.toggle_toc.connect(self.toggle_toc)
+        self.bridge.show_search.connect(self.show_search)
+        self.bridge.search_result_not_found.connect(self.search_result_not_found)
+        self.bridge.find_next.connect(self.find_next)
         self.bridge.toggle_bookmarks.connect(self.toggle_bookmarks)
         self.bridge.toggle_inspector.connect(self.toggle_inspector)
         self.bridge.toggle_lookup.connect(self.toggle_lookup)
@@ -455,6 +477,7 @@ class WebView(RestartingWebEngineView):
         self.bridge.toggle_full_screen.connect(self.toggle_full_screen)
         self.bridge.ask_for_open.connect(self.ask_for_open)
         self.bridge.selection_changed.connect(self.selection_changed)
+        self.bridge.autoscroll_state_changed.connect(self.autoscroll_state_changed)
         self.bridge.view_image.connect(self.view_image)
         self.bridge.copy_image.connect(self.copy_image)
         self.bridge.overlay_visibility_changed.connect(self.overlay_visibility_changed)
@@ -501,7 +524,7 @@ class WebView(RestartingWebEngineView):
         # TODO: Add UI for this
         ss = vprefs['session_data'].get('zoom_step_size') or 20
         amt = (ss / 100) * steps
-        self._page.setZoomFactor(self._page.zoomFactor() + amt)
+        self._page.setZoomFactor(max(0.25, min(self._page.zoomFactor() + amt, 5)))
 
     def render_process_died(self):
         if self.dead_renderer_error_shown:
@@ -532,9 +555,12 @@ class WebView(RestartingWebEngineView):
     def on_bridge_ready(self):
         f = QApplication.instance().font()
         fi = QFontInfo(f)
+        family = f.family()
+        if family in ('.AppleSystemUIFont', 'MS Shell Dlg 2'):
+            family = 'system-ui'
         ui_data = {
             'all_font_families': QFontDatabase().families(),
-            'ui_font_family': f.family(),
+            'ui_font_family': family,
             'ui_font_sz': '{}px'.format(fi.pixelSize()),
             'show_home_page_on_ready': self.show_home_page_on_ready,
             'system_colors': system_colors(),
@@ -544,6 +570,12 @@ class WebView(RestartingWebEngineView):
         for func, args in iteritems(self.pending_bridge_ready_actions):
             getattr(self.bridge, func)(*args)
 
+    def on_view_created(self, data):
+        self.view_created.emit(data)
+
+    def on_content_file_changed(self, data):
+        self.current_content_file = data
+
     def start_book_load(self, initial_position=None):
         key = (set_book_path.path,)
         self.execute_when_ready('start_book_load', key, initial_position, set_book_path.pathtoebook)
@@ -626,5 +658,8 @@ class WebView(RestartingWebEngineView):
     def trigger_shortcut(self, which):
         self.execute_when_ready('trigger_shortcut', which)
 
+    def show_search_result(self, sr):
+        self.execute_when_ready('show_search_result', sr)
+
     def palette_changed(self):
         self.execute_when_ready('set_system_palette', system_colors())
